#!/bin/ksh
#
# File: checkup.sh
# Checkup is intended to check for various problems in a database's schemas ans parameters
# that need to be addressed as new problems are found they are added to checkup.
# Checkup also recompiles any invalid objects before it reports invalid objects.
# The backup user runs the SQL.
# Usage: checkup.sh [-all|<sid>] [-v]
# -all (Default) = Preforms checkup on all sids in crontab
# <sid> = Preforms checkup only on the specified sid.
# -v = Sends email to members of sid_email.txt even if there are no warnings.
# Note: sid_email addresses can be specified to receieve messages for ALL instances or just specific sids. 
# Author: Doug Jackson  5/17/2002.

#--------------------------------------------------------------------
#- Send Mail
#--------------------------------------------------------------------
proc_send_mail()
{
  # Check $MAILOUT for Warning and send mail if it exist or if VERBOSE=YES
  if [ ! -z "`grep WARNING: $MAILOUT`" ]
  then
    WARNING="YES"
    SUBJECT="WARNING found in Checkup Report for $SID on $HOST `date`"
  else
    WARNING="NO"
    SUBJECT="Checkup Report for $SID on $HOST `date`"
  fi
  if [ "$WARNING" = "YES" ] || [ "$VERBOSE" = "YES" ] || [ "$STATUS" = "ERROR" ]
  then
    echo "Verbose = $VERBOSE" >> $MAILOUT
    echo "Warning = $WARNING" >> $MAILOUT
    echo "STATUS = $STATUS" >> $MAILOUT
    # Comment out strings that mail can not tollarate.
    cat $MAILOUT | sed 's/^call/#call/g' | sed 's/^BEGIN/#BEGIN/g' | sed 's/^begin/#begin/g' > $TEMPFILE1
    # Send to recipients for ALL sids
    for EACH in `cat $EMAIL_LIST | grep -v "^#" | grep "^ALL:" | sort | uniq | cut -d: -f2`
    do
      mail -s "$SUBJECT" $EACH < $TEMPFILE1
    done
    # Send to recipients for current sid
    for EACH in `cat $EMAIL_LIST | grep -v "^#" | grep "^$SID:" | sort | uniq | cut -d: -f2`
    do
      mail -s "$SUBJECT" $EACH < $TEMPFILE1
    done
  fi
}

#--------------------------------------------------------------------
#- Clean Up
#--------------------------------------------------------------------
proc_clean_up()
{
  if [ -f $MAILOUT ] ; then rm -f $MAILOUT ; fi
  if [ -f $TEMPFILE1 ] ; then rm -f $TEMPFILE1 ; fi
  if [ -f checkup_recompile_invalid.sql ] ; then rm -f checkup_recompile_invalid.sql ; fi
}

#################################################################################################################
### MAIN PROGRAM
#################################################################################################################

SCRIPT_DIR=`dirname $0`
if [ "$SCRIPT_DIR" = "." ]
then
  SCRIPT_DIR=`pwd`
fi

#Configuration
# BACKUPDIR is only used with -disk parameter and it will be appended with the ORACLE_SID
EMAIL_LIST=$SCRIPT_DIR/sid_email.txt
ORATAB=/etc/oratab
BACKUPUSR=$SCRIPT_DIR/backup_users.txt

MAILOUT=$SCRIPT_DIR/checkup_mail.tmp
TEMPFILE1=$SCRIPT_DIR/checkup1.tmp
STATUS="Normal"
HOST=`hostname`

# Make the dir with the SQL script the current directory
cd $SCRIPT_DIR

# Check for oratab file existence
if [[ ! -f $ORATAB ]]
then
  echo "ERROR: $ORATAB DOES NOT EXIST! checkup.sh failed."
  echo "ERROR: $ORATAB DOES NOT EXIST! checkup.sh failed." >> $MAILOUT
  STATUS="ERROR"
  proc_send_mail
  proc_clean_up
  exit 1
fi

if [ ! -f $EMAIL_LIST ]
then
  echo "ERROR: $EMAIL_LIST DOES NOT EXIST! checkup.sh failed."
  echo "ERROR: $EMAIL_LIST DOES NOT EXIST! checkup.sh failed." >> $MAILOUT
  STATUS="ERROR"
  proc_send_mail
  proc_clean_up
  exit 1
fi

if [ -z "`echo $* | grep -i '\-v'`" ]
then
  VERBOSE=NO
else 
  VERBOSE=YES
fi

if [ -z "$1" ] || [ "$1" = "-v" ] || [ "$1" = "-all" ]
then
  for SID in `cat $ORATAB | grep : | grep -v "^#" | grep -v "^\*" | grep -v xxx: | grep -v lsnr: | cut -d: -f1`
  do
    SID_LIST=`echo $SID_LIST $SID`
  done
else 
  SID=$1
  # Verify this SID is in the ORATAB
  if [ -z "`grep ^${SID}: $ORATAB`" ]
  then
    echo ERROR: SID not found in oratab: $SID
    echo ERROR: SID not found in oratab: $SID >> $MAILOUT
    STATUS="ERROR"
    proc_send_mail
    proc_clean_up
    exit 1
  else
    SID_LIST=$SID
  fi
fi

# Source the Oracle users's profile to set the path
. ~/.profile noup > /dev/null

# Get HOST OS
# AIX
OS="AIX"
OS_VER=`oslevel -r 2>/dev/null` 
if [ ! -z "$OS_VER" ]
then
  OS_VER=`echo AIX $OS_VER`
else
  # Linux
  OS="LINUX"
  OS_VER=`cat /proc/version | cut -d\( -f1 2>/dev/null` 
fi
if [ -z "$OS_VER" ]
then
  # Sun
  OS="SUN"
  OS_VER=`uname -rs 2>/dev/null` 
fi

# Step through each SID
for SID in $SID_LIST
do

  STATUS="normal"

  # Set the environment
  export ORACLE_SID=$SID
  export ORAENV_ASK=NO
  . oraenv
  . set_lib.sh

  # Initialize Mail file
  echo "Host: $HOST" > $MAILOUT
  echo "Instance: $ORACLE_SID" >> $MAILOUT
  echo "OS: $OS_VER" >> $MAILOUT
  echo "Storage: $STORAGE" >> $MAILOUT
  echo "`date`" >> $MAILOUT
  echo "" >> $MAILOUT

  # Running backup_config.sh on ORACLE_SID in order to generate fresh dblinks_tnsnames.ora output.
  if [ -x $SCRIPT_DIR/backup_config.sh ]
  then
    $SCRIPT_DIR/backup_config.sh $SID
    if [ -r /oracle_share/backup_configs/$ORACLE_SID/dblinks_tnsnames.ora ]
    then
      if [ ! -z "`grep ERROR: /oracle_share/backup_configs/$ORACLE_SID/dblinks_tnsnames.ora`" ]
      then
        echo "Warnings" >>$MAILOUT
        echo "--------------------------------------------------------------------------------" >>$MAILOUT
        echo "Invalid Database Links:" >>$MAILOUT
        grep "^## " /oracle_share/backup_configs/$ORACLE_SID/dblinks_tnsnames.ora |sed 's/#//g' | sed 's/ ERROR://g'  >>$MAILOUT
      fi
    fi
  fi

  # Get the backup user and password for this SID
  BACKUPUSER=`grep $HOST:$SID: $BACKUPUSR | grep -v ^# | cut -d: -f3`
  BACKUPPASS=`grep $HOST:$SID: $BACKUPUSR | grep -v ^# | cut -d: -f4`

  # The database must be up
  if [ -z "`ps -ef | grep -v grep | grep ora_smon_$ORACLE_SID`" ]
  then
    echo "Database $ORACLE_SID was not up. checkup not preformed..." >> $MAILOUT
    STATUS="ERROR"
    continue
  fi

  # Get Storage
  STORAGE=`mount | grep -i "\/$ORACLE_SID " | cut -d\/ -f1 | sed s/_san//g | sed s/://g `
  if [ -z "$STORAGE" ]
  then
    STORAGE="unknown"  
  fi

  # Verify valid user/password with sysdba privilege.
sqlplus -s /nolog << ENDIT > $TEMPFILE1
connect $BACKUPUSER/$BACKUPPASS as sysdba
exit
ENDIT
  if [ ! -z "`grep 'ORA-' $TEMPFILE1`" ]
  then
    echo "ERROR: Can not connect to $ORACLE_SID. checkup failed." >> $MAILOUT
    grep "ORA-" $TEMPFILE1 >> $MAILOUT
    STATUS="ERROR"
    continue
  fi

  # Attempt to recompile any invalid objects as SYS before running checkup.sql
sqlplus -s /nolog << ENDIT > /dev/null
connect $BACKUPUSER/$BACKUPPASS as sysdba
set head off
spool $SCRIPT_DIR/checkup_recompile.sql
select 'alter '||decode(object_type,'PACKAGE BODY','PACKAGE',object_type) ||' '||CHR(34)||owner||CHR(34)||'.'||CHR(34)||object_name||CHR(34)||decode(object_type,'PACKAGE BODY', ' COMPILE BODY;',' COMPILE;')
  from sys.dba_objects where status='INVALID';
spool off
@$SCRIPT_DIR/checkup_recompile.sql
exit
ENDIT

  # Run checkup.sql
sqlplus -s /nolog << ENDIT >> $MAILOUT
connect $BACKUPUSER/$BACKUPPASS as sysdba
@$SCRIPT_DIR/checkup.sql "$OS" "$OS_VER" "$STORAGE"
exit
ENDIT

  proc_send_mail
 
# End SID Loop
done

proc_clean_up
exit 0
# end of command file
